import 'package:flutter/material.dart';
import 'package:viqcore/viqcore.dart';

class ThemeObject {
  static const String _storedThemeId = '_ThemeObjectId_';

  static MaterialColor get selectedColor =>
      _selectedColorIndex > -1 ? Colors.primaries[_selectedColorIndex] : null;

  static int _selectedColorIndex = -1;
  static ThemeData _selectedTheme;
  static void showColorPicker(BuildContext context) {
    ColorSwatchPicker.colorSwatch = selectedColor;
    ColorSwatchPicker.showColorPicker(
        context: context,
        //onColorChange: (val) {},
        onChange: (color) {
          _storeTheme(color);
        });
  }

  static ThemeData getTheme(ThemeData defaultTheme) {
    _selectedTheme ??=
        selectedColor == null ? defaultTheme : ThemeData(primarySwatch: selectedColor);
    return _selectedTheme;
  }

  static Future<bool> init() async {
    await _loadStoredTheme();
    return Future.value(true);
  }

  static Future _storeTheme(MaterialColor color) async {
    await Prefs.setInt(_storedThemeId, Colors.primaries.indexOf(color));
    _selectedTheme = null;
    await _loadStoredTheme();
    App.refresh();
  }

  static Future _loadStoredTheme() async {
    _selectedColorIndex = await Prefs.getIntF(_storedThemeId, -1);
  }
}
