import 'package:flutter/foundation.dart';
import 'package:viqcore/viqcore.dart';
import 'package:meta/meta.dart';

class UserObject extends DataObject implements ChangeNotifier {
  ChangeNotifier _notifier = ChangeNotifier();

  //@override
  void addListener(listener) => _notifier?.addListener(listener);
  bool get hasListeners => _notifier?.hasListeners;
  @protected
  void notifyListeners() => _notifier?.notifyListeners();
  void removeListener(listener) => _notifier?.removeListener(listener);

  UserState get userState => _userState;
  @protected
  set userState(UserState state) => _userState = state;
  UserState _userState = UserState.NoUser;

  UserRole get userRole => _userRole;
  @protected
  set userRole(UserRole role) => _userRole = role;
  UserRole _userRole = UserRole.Anonymous;

  String get token => _token;
  @protected
  set token(String value) {
    _token = value;
    Prefs.setString('token', value);
  }

  String _token;

  @override
  Future<bool> init() async {
    super.init();
    _notifier ??= ChangeNotifier();
    _userState = UserState.NoUser;
    token = await Prefs.getStringF('token');
    await login();
    return Future.value(true);
  }

  @override
  void dispose() {
    userState = UserState.NoUser;
    userRole = UserRole.Anonymous;
    _notifier?.dispose();
    _notifier = null;
    super.dispose();
  }

  Future<DataResponse> login([String userId, String password]) async {
    _userState = UserState.LoggingInProgress;
    // exclusive or
    if (isNullOrEmpty(userId) ^ isNullOrEmpty(password)) {
      _userState = UserState.NoUser;
      return DataResponse.failed('Empty userId or password');
    }

    WebClientResult result;
    if (isNullOrEmpty(userId) && isNullOrEmpty(password)) {
      result = await onLoginToken();
    } else {
      result = await onLoginUserPassword(userId, password);
    }
    _handleLogin(result);
    return DataResponse.fromWebClientResult(result);
  }

  void logout() {
    _userRole = UserRole.Anonymous;
    _userState = UserState.NoUser;
    this.clear();
    token = null;
  }

  void _handleLogin(WebClientResult result) {
    _userRole = UserRole.Anonymous;
    _userState = UserState.NoUser;
    if (result.ok) {
      _userState = UserState.UserLoggedIn;
      onLoginSuccess(result);
      _userState = UserState.UserLoggedIn;
    }
  }

  Future<DataResponse> testLogin(String userId, String password,
      [int delayedInSeconds = 2]) async {
    _userState = UserState.LoggingInProgress;

    await Future.delayed(Duration(seconds: delayedInSeconds));

    _userState = UserState.NoUser;

    if (userId == 'test' && password == 'password') {
      _userState = UserState.UserLoggedIn;
      return DataResponse.success();
    }
    return DataResponse.failed('Invalid user id or password');
  }

  Future<WebClientResult> onLoginToken() async {
    return WebClientResult.failed('Not support');
  }

  Future<WebClientResult> onLoginUserPassword(
      String userId, String password) async {
    return WebClientResult.failed('Not implemented');
  }

  @mustCallSuper
  void onLoginSuccess(WebClientResult result) {
    fill(result.data);
    _userRole = UserRole.Anonymous;
    if (onDetermineSuperAdminRole()) {
      _userRole = UserRole.SuperAdmin;
    } else if (onDetermineAdminRole()) {
      _userRole = UserRole.Admin;
    }
  }

  bool onDetermineSuperAdminRole() {
    return false;
  }

  bool onDetermineAdminRole() {
    return false;
  }
}

/// Enumeration of possible states for an user.
enum UserState { NoUser, LoggingInProgress, UserLoggedIn }
enum UserRole { Anonymous, Admin, SuperAdmin }
