import 'dart:async';
import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:image_picker/image_picker.dart';
import 'package:image_cropper/image_cropper.dart';
import 'package:validators/validators.dart' as validators;
import 'dart:io';
import 'package:viqcore/viqcore.dart';

class FormBuilderImageAvatarField extends StatefulWidget {
  final String attribute;
  //final List<FormFieldValidator> validators;
  final String initialValue;
  final bool readonly;
  final InputDecoration decoration;
  final ValueChanged onChanged;
  final ValueTransformer valueTransformer;
  //final String label;
  final bool enableCropping;

  //this.initialsText = const Text(''),
  final double radius; // = 50.0,
  final double borderWidth; // = 0.0,
  final Color borderColor; // = Colors.white,
  final Color backgroundColor; // = Colors.white,
  final double elevation; // = 0.0,
  // this.showInitialTextAbovePicture = false,
  final GestureTapCallback onTap;
  //final Color foregroundColor;

  FormBuilderImageAvatarField({
    @required this.attribute,
    //@required this.label,
    this.initialValue,
    this.readonly = false,
    this.decoration = const InputDecoration(border: InputBorder.none),
    this.onChanged,
    this.valueTransformer,
    this.enableCropping = true,
    this.radius = 50.0,
    this.borderWidth = 0.0,
    this.borderColor = Colors.white,
    this.backgroundColor = Colors.white,
    this.elevation = 0.0,
    this.onTap,
    //this.foregroundColor = Colors.transparent,
  });

  @override
  _FormBuilderImageAvatarFieldState createState() =>
      _FormBuilderImageAvatarFieldState();
}

class _FormBuilderImageAvatarFieldState
    extends State<FormBuilderImageAvatarField> {
  bool _readonly = false;
  final GlobalKey<FormFieldState> _fieldKey = GlobalKey<FormFieldState>();

  File _imageFile;
  String _imageUrl;
  bool _dirty = false;

  @override
  void initState() {
    registerFieldKey();
    _readonly =
        (FormBuilder.of(context)?.readonly == true) ? true : widget.readonly;

    if (widget.initialValue != null) {
      if (validators.isURL(widget.initialValue)) {
        _imageUrl = widget.initialValue;
      } else {
        Uri uri = Uri(path: widget.initialValue);
        _imageFile = File.fromUri(uri);
      }
    }
    super.initState();
  }

  registerFieldKey() {
    if (FormBuilder.of(context) != null)
      FormBuilder.of(context).registerFieldKey(widget.attribute, _fieldKey);
  }

  dynamic _transform(dynamic value) {
    if (value == null) return null;
    if (widget.valueTransformer != null) {
      return widget.valueTransformer(_imageFile);
    }
    return base64Encode(_imageFile.readAsBytesSync());
  }

  @override
  Widget build(BuildContext context) {
    return FormField(
        key: _fieldKey,
        enabled: !_readonly,
        onSaved: (val) {
          if (_dirty) {
            if (_imageFile != null) {
              var value = _transform(_imageFile);
              FormBuilder.of(context)
                    ?.setAttributeValue(widget.attribute, value);
            }
          }
        },
        builder: (FormFieldState<dynamic> field) {
          return InputDecorator(
            decoration: widget.decoration.copyWith(
              enabled: !_readonly,
              errorText: field.errorText,
            ),
            child: Column(
              children: <Widget>[
                // Row(
                //   children: <Widget>[
                //     Text(
                //       widget.label,
                //       style: Theme.of(context).textTheme.caption,
                //       textAlign: TextAlign.start,
                //     ),
                //   ],
                // ),
                Row(
                  children: <Widget>[
                    Container(
                      child: _imageFile != null
                          ? CircularProfileAvatar(
                              _imageFile.path,
                              radius: widget.radius,
                              borderWidth: widget.borderWidth,
                              borderColor: widget.borderColor,
                              backgroundColor: widget.backgroundColor,
                              elevation: widget.elevation,
                              onTap: widget.onTap,
                              // foregroundColor: widget.foregroundColor,
                            )
                          : CircularProfileAvatar(
                              _imageUrl,
                              radius: widget.radius,
                              borderWidth: widget.borderWidth,
                              borderColor: widget.borderColor,
                              backgroundColor: widget.backgroundColor,
                              elevation: widget.elevation,
                              onTap: widget.onTap,
                              //   foregroundColor: widget.foregroundColor,
                            ),
                    ),
                    _readonly
                        ? Container()
                        : Column(
                            children: [
                              IconButton(
                                icon: Icon(Icons.image),
                                onPressed: () {
                                  _setImageFile(ImageSource.gallery);
                                },
                              ),
                              IconButton(
                                icon: Icon(Icons.camera),
                                onPressed: () {
                                  _setImageFile(ImageSource.camera);
                                },
                              ),
                              IconButton(
                                icon: Icon(Icons.clear),
                                onPressed: () {
                                  setState(() {
                                    _imageFile = null;
                                    _imageUrl = null;
                                    _dirty = true;
                                  });
                                  if (widget.onChanged != null) {
                                    widget.onChanged(null);
                                  }
                                },
                              )
                            ],
                          ),
                  ],
                ),
              ],
            ),
          );
        });
  }

  void _setImageFile(ImageSource source) async {
    File file = await ImagePicker.pickImage(source: source);
    if (file == null) return;
    if (widget.enableCropping) {
      file = await _cropImage(file);
    }
    setState(() {
      _imageUrl = null;
      _imageFile = file;
      _dirty = true;
    });
    if (widget.onChanged != null) {
      widget.onChanged(_transform(_imageFile));
    }
  }

  Future _cropImage(File image) async {
    File croppedFile = await ImageCropper.cropImage(
      sourcePath: image.path,
      ratioX: 1.0,
      ratioY: 1.0,
      maxWidth: 512,
      maxHeight: 512,
    );
    return croppedFile;
  }
}
