import 'package:flutter/material.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';

class MenuboardItem {
  final Color color;
  final IconData icon;
  final String label;
  final String tag;
  final String tooltip;
  MenuboardItem(this.tag,
      {this.label, this.icon, this.color = Colors.blue, this.tooltip});
}

enum MenuboardStyle { StaggerTile, AppButtonTile }

class MenuboardPage extends StatelessWidget {
  static const tag = '/menuboard';
  final String title;
//  final int column;
  //final String image;
  final BoxDecoration decoration;

  final List<MenuboardItem> items;
  final MenuboardStyle style;

  MenuboardPage(this.items,
      {this.style = MenuboardStyle.StaggerTile,
      this.title = "Menuboard",
      this.decoration});

  @override
  Widget build(BuildContext context) {
    return new Scaffold(
      appBar: new AppBar(
        title: new Text(title),
      ),
      body: Container(child: _buildTiles(items, style), decoration: decoration),
    );
  }

  Widget _buildTiles(List<MenuboardItem> items, MenuboardStyle style) {
    if (style == MenuboardStyle.AppButtonTile) return _buildAppMenuTiles(items);

    return _buildStaggerTiles(items);
  }

  Widget _buildAppMenuTiles(List<MenuboardItem> items) {
    return GridView.builder(
        itemCount: items.length,
        //shrinkWrap: true,
        gridDelegate:
            new SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: 3),
        itemBuilder: (BuildContext context, int index) {
          MenuboardItem item = items[index];

          return Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            children: <Widget>[
              Card(
                  color: item.color,
                  shape: RoundedRectangleBorder(
                      borderRadius: BorderRadius.circular(20)),
                  elevation: 5.0,
                  child: IconButton(
                    color: Colors.white,
                    icon: Icon(
                      item.icon,
                    ),
                    iconSize: 30,
                    tooltip: item.tooltip,
                    onPressed: () => {
                          //print(item.tag)
                          Navigator.of(context).pushNamed(item.tag)
                        },
                  )),
              Text(item.label)
            ],
          );
        });
  }

  Widget _buildStaggerTiles(List<MenuboardItem> items) {
    return new Padding(
        padding: const EdgeInsets.only(top: 4.0),
        child: new StaggeredGridView.count(
          crossAxisCount: 3,
          staggeredTiles: items.map((item) {
            return const StaggeredTile.fit(1);
          }).toList(),
          children: items.map((item) {
            return _MenuboardTile(item);
          }).toList(),
          mainAxisSpacing: 1.0,
          crossAxisSpacing: 1.0,
          padding: const EdgeInsets.all(4.0),
        ));
  }
}

// class _MenuboardTile extends StatelessWidget {
//   _MenuboardTile(this.item);

//   final MenuboardItem item;

//   @override
//   Widget build(BuildContext context) {
//     Widget result = new Card(
//         color: item.color,
//         child: new InkWell(
//             onTap: () => Navigator.of(context).pushNamed(item.tag),
//             child:
//                 Column(crossAxisAlignment: CrossAxisAlignment.start, children: [
//               Padding(
//                   padding: const EdgeInsets.all(4.0),
//                   child: Text(item.label,
//                       style: TextStyle(
//                         color: Colors.white,
//                         fontWeight: FontWeight.bold,
//                       ))),
//               Padding(
//                   padding: const EdgeInsets.all(4.0),
//                   child: Align(
//                       alignment: Alignment.bottomRight,
//                       child: Icon(
//                         item.icon,
//                         color: Colors.white,
//                         size: 80,
//                       ))),
//             ])));
//     if (item.tooltip != null) {
//       result = Tooltip(
//         message: item.tooltip,
//         child: result,
//       );
//     }
//     return result;
//   }
// }

class _MenuboardTile extends StatelessWidget {
  _MenuboardTile(this.item);

  final MenuboardItem item;

  @override
  Widget build(BuildContext context) {
    Widget result = new Card(
        color: item.color,
        child: new InkWell(
            onTap: () => Navigator.of(context).pushNamed(item.tag),
            child:
                Column(crossAxisAlignment: CrossAxisAlignment.center, children: [
              Padding(
                  padding: const EdgeInsets.all(4.0),
                  child: Icon(
                    item.icon,
                    color: Colors.white,
                    size: 50,
                  )),
              Padding(
                  padding: const EdgeInsets.all(4.0),
                  child: Align(
                      alignment: Alignment.bottomCenter,
                      child: Text(item.label,
                          style: TextStyle(
                            color: Colors.white,
                            fontWeight: FontWeight.bold,
                          )))),
            ])));
    if (item.tooltip != null) {
      result = Tooltip(
        message: item.tooltip,
        child: result,
      );
    }
    return result;
  }
}
