import 'package:flutter/material.dart';
import 'package:viqcore/viqcore.dart';

/// A button that will add editing options UI to a documentList document.
/// It creates a builder button with a contextial menu, that allows editing
/// or deleting of the document by passing in the documentList and the index
/// of the document to edit.
class DataActionsButton extends StatelessWidget {
  /// The DocumentList on which the button and forms that it invokes act.
  final DataListController dataListController;

  /// The index of document in documentList on which button acts.
  final int index;

  // /// A BoxDecoration for automatically generated forms
  // final BoxDecoration formDecoration;

  // /// A BoxDecoration for each field in a DocumentForm
  // final BoxDecoration formFieldDecoration;

  final WidgetBuilder dataFormEditBuilder;

  DataActionsButton(this.dataFormEditBuilder, this.dataListController,
      {@required this.index});//, this.formDecoration, this.formFieldDecoration});

  @override
  Widget build(BuildContext context) {
    return PopupMenuButton<int>(
        onSelected: (int action) {
          switch (action) {
            case 0:
              dataListController.openEditForm(context, dataFormEditBuilder, index: index);
              break;
            case 1:
              dataListController.delete(index);
          }
        },
        itemBuilder: (BuildContext context) => <PopupMenuItem<int>>[
              PopupMenuItem<int>(
                value: 0,
                child: Icon(Icons.edit),
              ),
              PopupMenuItem<int>(
                value: 1,
                child: Icon(Icons.delete),
              ),
            ]);
  }
}
