import 'package:flutter/material.dart';
import 'package:viqcore/viqcore.dart';

abstract class DataController extends ControllerMVC implements ChangeNotifier {
  DataController();

  BuildContext get context => stateMVC?.context;

  ChangeNotifier _notifier = ChangeNotifier();

  //@override
  void addListener(listener) => _notifier?.addListener(listener);
  bool get hasListeners => _notifier?.hasListeners;
  @protected
  void notifyListeners() => _notifier?.notifyListeners();
  void removeListener(listener) => _notifier?.removeListener(listener);

  String get title => this.runtimeType.toString();

  bool get inAsyncCall => _asyncCallStack.length > 0;

  List<String> _asyncCallStack = List<String>();
  Map<String, Flushbar> _flushbarMaps = Map<String, Flushbar>();
  void _flushbarShow(String title) {
    Flushbar fb = onGetAsyncLoadingFlushbar(title);
    if (fb.duration != null) {
      _asyncCallStack.add('_dummykey_');
      return;
    }
    String key = Uuid().generateV4();
    _asyncCallStack.add(key);
    if (fb.duration == null) {
      _flushbarMaps[key] = fb;
    }
    fb.show(this.stateMVC.context);
  }

  void _flushbarDismiss() {
    if (_asyncCallStack.length == 0) return;
    String key = _asyncCallStack.removeLast();
    if (!_flushbarMaps.containsKey(key)) return;
    Flushbar fb = _flushbarMaps[key];
    if (fb == null) return;
    fb.dismiss();
  }

  void _inAsyncCallStart(String title) {
    _flushbarShow(title);
    onAsyncCallStart();
  }

  void _inAsyncCallEnd() {
    _flushbarDismiss();
    onAsyncCallEnd();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }

  @override
  void initState() {
    super.initState();
    _notifier ??= ChangeNotifier();
  }

  @override
  void dispose() {
    if (_notifier != null && !_notifier.hasListeners) {
      _notifier.dispose();
      _notifier = null;
    }
    _flushbarMaps?.clear();
    _flushbarMaps = null;
    super.dispose();
  }

  @override
  void refresh() {
    super.refresh();
  }

  Future<DataResponse> asyncInProgress(Future<DataResponse> response,
      {String title}) {
    _inAsyncCallStart(title);
    response.whenComplete(() => _inAsyncCallEnd());
    response.then((response) => handleDataResponse(response));
    return response;
  }

  void handleDataResponse(DataResponse response) {
    if (response?.ok ?? true) return;
    onDataResponseError(response);
  }

  void onAsyncCallStart() {
    this.refresh();
  }

  void onAsyncCallEnd() {
    this.refresh();
  }

  void onDataResponseError(DataResponse response) {
    FlushbarHelper.createError(
            message: response.error,
            title: translation(context, LangSysIds.dataError))
        .show(context);
  }

  Flushbar onGetAsyncLoadingFlushbar(String title) {
    title = title ?? translation(context, LangSysIds.waitingForResult);
    //return FlushbarHelper.createLoading(
    //     message: '$title...',
    //     linearProgressIndicator: LinearProgressIndicator(),
    //     duration: null);
    return Flushbar(
      //title: title,
      message: '$title...',
      duration: null,
      icon: Icon(
        Icons.cloud_upload,
        color: Colors.blue[300],
      ),
      showProgressIndicator: true,
      overlayBlur: 0.5,
      overlayColor: Colors.blue[200].withOpacity(0.2),
      //progressIndicatorController: progressIndicatorController,
      //progressIndicatorBackgroundColor: progressIndicatorBackgroundColor,
    );
  }
}
