import 'package:flutter/material.dart';
import 'package:viqcore/viqcore.dart';

class DataFormController<T extends DataObject> extends DataController {
  final T dataObject;
  final bool isViewOnly;
  Map<String, dynamic> updatedValues;

  ///The controller is responsible in managing the [dataObject], hence a [dataObject] instance must be provided.
  ///In case it is requested by [dataListController], then the [dataObject] can be omitted for new record purpose.
  DataFormController(
      {@required String title, this.dataObject, this.isViewOnly = false})
      : assert(dataObject != null);

  void save() {
    if (onSave()) {
      _save();
    }
  }

  Future<DataResponse> _save() async {
    if (updatedValues != null) dataObject.updateValues(updatedValues);
    Future<DataResponse> response;
    var isNew = dataObject.isNew;
    if (isNew) {
      response = dataObject.create();
    } else {
      response = dataObject.update();
    }
    asyncInProgress(response).then((_) {
      if (_.ok) onSaveSuccess(isNew);
    });
    return response;
  }

  Future<DataResponse> delete(DataObject dataObject) async {
    return await dataObject.delete();
  }

  void onSaveSuccess(bool isNew) {}
  bool onSave() {
    return true;
  }
}

mixin FlutterFormMixin<T extends DataObject> on DataFormController<T> {
  GlobalKey<FormBuilderState> get formKey => _formKey ?? _newFormKey();
  GlobalKey<FormBuilderState> _formKey;
  GlobalKey<FormBuilderState> _newFormKey() {
    _formKey = GlobalKey<FormBuilderState>();
    return _formKey;
  }

  @override
  void dispose() {
    _formKey = null;
    super.dispose();
  }

  void save() {
    formKey.currentState.save();
    if (formKey.currentState.validate()) {
      this.updatedValues = formKey.currentState.value;
      _save();
    }
  }
}

mixin BasicFormMixin<T extends DataObject> on DataFormController<T> {
  GlobalKey<FormState> get formKey => _formKey ?? _newFormKey();
  GlobalKey<FormState> _formKey;
  GlobalKey<FormState> _newFormKey() {
    _formKey = GlobalKey<FormState>();
    return _formKey;
  }

  @override
  void dispose() {
    _formKey = null;
    super.dispose();
  }

  void save() {
    this.updatedValues ??= {};
    formKey.currentState.save();
    if (formKey.currentState.validate()) {
      _save();
    }
  }
}
