import 'package:flutter/material.dart';
import 'package:viqcore/viqcore.dart';

/// A dynamically generate form that allows creating new documents or
/// editing existing documents in a DocumentList. Specifying an index
/// for a document in a DocumentList causes the form to be in edit mode,
/// otherwise it will create a new document.
class DataFormScaffold extends StatefulWidget {
  final DataFormController dataFormController;

  /// Adds widgets to the titlebar, typically IconButtons.
  final List<Widget> additionalActions;

  final WidgetBuilder dataFormBuilder;

  final IconData icon;
  final BoxDecoration decoration;

  DataFormScaffold({
    @required this.dataFormController,
    @required this.dataFormBuilder,
    this.icon,
    this.additionalActions,
    this.decoration,
  }); //, this.decoration, this.fieldDecoration});

  @override
  _DataFormScaffoldState createState() => _DataFormScaffoldState();
}

class _DataFormScaffoldState extends StateMVC<DataFormScaffold> {
  @override
  void initState() {
    add(widget.dataFormController);
    super.initState();
    //formKey = widget.dataFormController.formKey;
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> listTitle = [];
    var iconData = widget.icon ?? widget.dataFormController.isViewOnly
        ? Icons.pageview
        : widget.dataFormController.dataObject.isNew ? Icons.add : Icons.edit;

    listTitle.add(Icon(iconData));
    listTitle.add(Text(widget.dataFormController.title));

    List<Widget> actions = <Widget>[];
    if (widget.additionalActions != null) {
      actions += widget.additionalActions;
    }

    if (!widget.dataFormController.isViewOnly) {
      actions.add(IconButton(
          icon: Icon(Icons.save),
          onPressed: () => widget.dataFormController.save()));
    }

    return ChangeNotifierProvider<DataFormController>.value(
        notifier: widget.dataFormController,
        child: Scaffold(
          appBar: AppBar(
            title: Row(
              children: listTitle,
            ),
            actions: actions,
          ),
          body: Container(
           // child:widget.dataFormBuilder(context),
            child: Builder(
              builder: (BuildContext context) =>
                  widget.dataFormBuilder(context),
            ),            
            decoration: widget.decoration,
          ),
        ));
  }
}
