import 'package:flutter/material.dart';
import 'package:viqcore/viqcore.dart';

class DataListScaffold extends StatefulWidget {
  /// A box decoration, that, if supplied will be applied to the DataListView.
  final BoxDecoration decoration;

  /// A call back function for building custom widgets for the DataListView,
  /// rather than the default ListTile. Like a normal builder, it receives a
  /// integer as an index into the associated DataList and should return a
  /// widget.
  /// Widget listItemBuilder(int index) {/* create and return a custom widget
  /// for documentList[index]*/}
  final IndexedWidgetBuilder listItemBuilder;
  final IndexedWidgetBuilder listHeaderBuilder;

  /// A widget to display when the DataListView is empty (when the
  /// DataList.length == 0)
  final Widget emptyListWidget;

  /// Optional string to pass to pass to the AddDocumentFloatingActionButton.
  /// If supplied, the fab will use an extended fab, with the label.
  final String addActionLabel;

  /// Adds widgets to the titlebar, typically IconButtons.
  final List<Widget> additionalActions;

  final WidgetBuilder dataFormEditBuilder;

  final DataListController dataListController;

  DataListScaffold({
    @required this.dataListController,
    @required this.listItemBuilder,
    this.listHeaderBuilder,
    this.emptyListWidget,
    this.dataFormEditBuilder,
    this.decoration,
    this.addActionLabel,
    this.additionalActions,
  });

  _DataListScaffoldState createState() => _DataListScaffoldState();
}

class _DataListScaffoldState<T extends DataListController>
    extends StateMVC<DataListScaffold> {
  @override
  void initState() {
    super.initState();
    this.add(widget.dataListController);
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> actions = <Widget>[];
    if (widget.dataListController.sortingFields != null) {
      actions.add(DataListSortButton(widget.dataListController));
    }
    if (widget.additionalActions != null) {
      actions += widget.additionalActions;
    }

    return
        //ScopedModel<ObjectModel<DataListController>>(
        //  model: widget.dataListController.model,
        ChangeNotifierProvider<DataListController>.value(
      notifier: widget.dataListController,
      child: Scaffold(
        appBar: AppBar(
          title: Text(widget.dataListController.title),
          actions: actions.length > 0 ? actions : null,
        ),
        body: Container(
          child: DataListView(
              listItemBuilder: widget.listItemBuilder,
              listHeaderBuilder: widget.listHeaderBuilder,
              emptyListWidget: widget.emptyListWidget),
          decoration: widget.decoration,
        ),
        floatingActionButton: widget.dataFormEditBuilder == null
            ? null
            : AddDataFloatingActionButton(
                dataFormEditBuilder: widget.dataFormEditBuilder,
                addActionLabel: widget.addActionLabel,
              ),
      ),
    );
  }
}
