import 'package:flutter/material.dart';
import 'package:viqcore/viqcore.dart';

/// A button that adds sorting to a DocumentList. Useful for adding
/// sorting functionality to a DocumentListView.
class DataListSortButton extends StatefulWidget {
  /// The DocumentList to be sorted
  final DataListController dataListController;

  DataListSortButton(this.dataListController);

  @override
  _DataListSortButtonState createState() => _DataListSortButtonState();
}

class _DataListSortButtonState extends State<DataListSortButton> {
  SortOrder currentSortOrder = SortOrder.ascending;
  String currentFieldName = "";
  @override
  Widget build(BuildContext context) {
    return PopupMenuButton<String>(
        icon: Icon(Icons.sort),
        onSelected: (String fieldName) {
          //toggle the sort if the same fieldname was selected multiple times
          if (fieldName == currentFieldName) {
            currentSortOrder == SortOrder.ascending
                ? currentSortOrder = SortOrder.descending
                : currentSortOrder = SortOrder.ascending;
          } else {
            currentSortOrder = SortOrder.ascending;
          }
          widget.dataListController
              .sortByField(fieldName, sortOrder: currentSortOrder);
          currentFieldName = fieldName;
        },
        itemBuilder: (BuildContext context) {
          List<String> unsortable = ["image", "latlong"];
          List<PopupMenuItem<String>> fieldButtons = [];
          widget.dataListController.sortingFields.forEach((String fieldName, String label) {
            bool addToList = true;
            unsortable.forEach((String s) {
              if (fieldName.toLowerCase().endsWith(s)) {
                addToList = false;
              }
            });
            if (addToList) {
              fieldButtons.add(PopupMenuItem<String>(
                  value: fieldName,
                  child: ListTile(
                    title: Text(label),
                  )));
            }
          });
          return fieldButtons;
        });
  }
}
