///
/// Copyright (C) 2018 Andrious Solutions
///
/// This program is free software; you can redistribute it and/or
/// modify it under the terms of the GNU General Public License
/// as published by the Free Software Foundation; either version 3
/// of the License, or any later version.
///
/// You may obtain a copy of the License at
///
///  http://www.apache.org/licenses/LICENSE-2.0
///
///
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
///          Created  24 Dec 2018
///

import 'dart:async' show Future;

import 'package:flutter/foundation.dart' show Key, mustCallSuper, required;

import 'package:flutter/material.dart';

import 'package:viqcore/viqcore.dart';

import 'package:flutter/services.dart';
import 'package:flutter_native_timezone/flutter_native_timezone.dart';
import 'package:time_machine/time_machine.dart';

import 'package:package_info/package_info.dart' show PackageInfo;

/// Highlights UI while debugging.
import 'package:flutter/rendering.dart' as debugPaint;

import 'package:flutter_localizations/flutter_localizations.dart';

typedef CreateView = AppView Function();

class App extends AppMVC {
  // You must supply a 'View.'
  factory App(CreateView createVW,
      {ControllerMVC con,
      Key key,
      Widget loadingScreen = const _SimpleLoadingScreen()}) {
    // Supply a 'Controller' if need be.
    _this ??= App._(createVW, con, key, loadingScreen);
    return _this;
  }

  // Make only one instance of this class.
  static App _this;

  App._(CreateView createVW, ControllerMVC con, Key key, this.loadingScreen)
      : super(con: con, key: key) {
    _vw = createVW();
    App.createVW = () {
      if (hotLoad) _vw = createVW();
      return _vw;
    };
  }

  static AppView _vw;
  static AsyncSnapshot get snapshot => _snapshot;
  static AsyncSnapshot _snapshot;
  final Widget loadingScreen;
  static CreateView createVW;
  static bool hotLoad = false;
  static UserObject get user => _vw.user;

  @override
  Widget build(BuildContext context) {
    //Assets.init(context);
    App._context = context;
    return FutureBuilder<bool>(
      future: init(),
      // builder: (_, snapshot) {
      //   Widget widget = loadingScreen == null ||
      //           snapshot.connectionState == ConnectionState.done
      //       ? _AppWidget(snapshot)
      //       : loadingScreen;
      //   return widget;
      // },
      builder: (_, snapshot) {
        return snapshot.connectionState == ConnectionState.done
            ? snapshot.data ? _AppWidget(snapshot) : loadingScreen
            : loadingScreen;
      },
    );
  }

  @override
  void initApp() {
    super.initApp();
    _vw.con.initApp();
  }

  @override
  Future<bool> init() async {
    if (!hotLoad) {
      _packageInfo = await PackageInfo.fromPlatform();
    }
    _isInit = await super.init();
    if (_isInit) _isInit = await _vw.init();
    return _isInit;
  }

  static void refresh() {
    _vw.refresh();
  }

  @mustCallSuper
  void dispose() {
    _context = null;
    _scaffold = null;
    super.dispose();
  }

  /// Determine if the App initialized successfully.
  static bool get isInit => _isInit;
  static bool _isInit = false;

  static GlobalKey<NavigatorState> get navigatorKey => _vw.navigatorKey;

  static Map<String, WidgetBuilder> get routes => _vw.routes;

  static String get initialRoute => _vw.initialRoute;

  static RouteFactory get onGenerateRoute => _vw.onGenerateRoute;

  static RouteFactory get onUnknownRoute => _vw.onUnknownRoute;

  static List<NavigatorObserver> get navigatorObservers =>
      _vw.navigatorObservers;

  static TransitionBuilder get builder => _vw.builder;

  static String get title => _vw.title;

  static GenerateAppTitle get onGenerateTitle => _vw.onGenerateTitle;

  // static ThemeData get theme => _vw.theme ?? App._getTheme();

  static Color get color => _vw.color;

  // static Locale get locale => _vw.locale;

  // static Iterable<LocalizationsDelegate<dynamic>> get localizationsDelegates =>
  //     _vw.localizationsDelegates;

  static LocaleResolutionCallback get localeResolutionCallback =>
      _vw.localeResolutionCallback;

  //static Iterable<Locale> get supportedLocales => _vw.supportedLocales;

  static bool get debugShowMaterialGrid => _vw.debugShowMaterialGrid;

  static bool get showPerformanceOverlay => _vw.showPerformanceOverlay;

  static bool get checkerboardRasterCacheImages =>
      _vw.checkerboardRasterCacheImages;

  static bool get checkerboardOffscreenLayers =>
      _vw.checkerboardOffscreenLayers;

  static bool get showSemanticsDebugger => _vw.showSemanticsDebugger;

  static bool get debugShowCheckedModeBanner => _vw.debugShowCheckedModeBanner;

  static BuildContext get context => _context;
  static BuildContext _context;

  static ScaffoldState _scaffold;

  // Application information
  static PackageInfo _packageInfo;

  static String get appName => _packageInfo?.appName;

  static String get packageName => _packageInfo?.packageName;

  static String get version => _packageInfo?.version;

  static String get buildNumber => _packageInfo?.buildNumber;

  /// Determines if running in an IDE or in production.
  static bool get inDebugger {
    var inDebugMode = false;
    // assert is removed in production.
    assert(inDebugMode = true);
    return inDebugMode;
  }

  static ScaffoldState get scaffold => App._getScaffold();

  static ScaffoldState _getScaffold() {
    if (_scaffold == null) _scaffold = Scaffold.of(_context, nullOk: true);
    return _scaffold;
  }

  void didChangeAppLifecycleState(AppLifecycleState state) {}
}

class _AppWidget extends StatefulWidget {
  _AppWidget(AsyncSnapshot snapshot, {Key key}) : super(key: key) {
    /// Supply the AsyncSnapshot
    App._snapshot = snapshot;
  }
  State createState() => App.createVW();
}

class AppView extends AppViewState<_AppWidget> {
  AppView(
      {this.key,
      @required this.home,
      AppController con,
      UserObject user,
      GlobalKey<NavigatorState> navigatorKey,
      Map<String, WidgetBuilder> routes,
      String initialRoute,
      RouteFactory onGenerateRoute,
      RouteFactory onUnknownRoute,
      List<NavigatorObserver> navigatorObservers,
      TransitionBuilder builder,
      String title,
      GenerateAppTitle onGenerateTitle,
      ThemeData theme,
      Color color,
      // Locale locale,
      // Iterable<LocalizationsDelegate<dynamic>> localizationsDelegates,
      LocaleResolutionCallback localeResolutionCallback,
      // Iterable<Locale> supportedLocales,
      bool debugShowMaterialGrid,
      bool showPerformanceOverlay,
      bool checkerboardRasterCacheImages,
      bool checkerboardOffscreenLayers,
      bool showSemanticsDebugger,
      bool debugShowCheckedModeBanner,
      bool debugPaintSizeEnabled,
      bool debugPaintBaselinesEnabled,
      bool debugPaintPointersEnabled,
      bool debugPaintLayerBordersEnabled,
      bool debugRepaintRainbowEnabled})
      : super(
          con: con,
          user: user ?? UserObject(),
          // themeObject: ThemeObject(),
          navigatorKey: navigatorKey,
          routes: routes,
          initialRoute: initialRoute,
          onGenerateRoute: onGenerateRoute,
          onUnknownRoute: onUnknownRoute,
          navigatorObservers: navigatorObservers,
          builder: builder,
          title: title,
          onGenerateTitle: onGenerateTitle,
          theme: theme,
          color: color,
          // locale: locale,
          //localizationsDelegates: localizationsDelegates,
          localeResolutionCallback: localeResolutionCallback,
          //supportedLocales: supportedLocales,
          debugShowMaterialGrid: debugShowMaterialGrid,
          showPerformanceOverlay: showPerformanceOverlay,
          checkerboardRasterCacheImages: checkerboardRasterCacheImages,
          checkerboardOffscreenLayers: checkerboardOffscreenLayers,
          showSemanticsDebugger: showSemanticsDebugger,
          debugShowCheckedModeBanner: debugShowCheckedModeBanner,
        );

  final Key key;
  final Widget home;

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: [
        ChangeNotifierProvider<UserObject>(builder: (_) => this.user),
      ],
      child: _materialApp(context),
    );
  }

  Widget _materialApp(BuildContext context) {
    return MaterialApp(
      key: key,
      navigatorKey: navigatorKey ?? onNavigatorKey(),
      home: home,
      routes: routes ?? onRoutes(),
      initialRoute: initialRoute ?? onInitialRoute(),
      onGenerateRoute: onGenerateRoute ?? onOnGenerateRoute(),
      onUnknownRoute: onUnknownRoute ?? onOnUnknownRoute(),
      navigatorObservers: navigatorObservers ?? onNavigatorObservers(),
      builder: builder ?? onBuilder(),
      title: title ?? onTitle(),
      onGenerateTitle: onGenerateTitle ?? onOnGenerateTitle(),
      color: color ?? onColor(),
      theme: _theme(theme), //theme ?? onTheme() ?? themeObject.theme,
      locale: LocaleObject.selectedLocale,
      localizationsDelegates: [
        GlobalMaterialLocalizations.delegate,
        GlobalWidgetsLocalizations.delegate,
        LocaleObject.delegate
      ],
      supportedLocales: LocaleObject.supportedLocales,
      localeResolutionCallback:
          localeResolutionCallback ?? onLocaleResolutionCallback(),
      debugShowMaterialGrid: debugShowMaterialGrid ?? onDebugShowMaterialGrid(),
      showPerformanceOverlay:
          showPerformanceOverlay ?? onShowPerformanceOverlay(),
      checkerboardRasterCacheImages:
          checkerboardRasterCacheImages ?? onCheckerboardRasterCacheImages(),
      checkerboardOffscreenLayers:
          checkerboardOffscreenLayers ?? onCheckerboardOffscreenLayers(),
      showSemanticsDebugger: showSemanticsDebugger ?? onShowSemanticsDebugger(),
      debugShowCheckedModeBanner:
          debugShowCheckedModeBanner ?? onDebugShowCheckedModeBanner(),
    );
  }

  /// During development, if a hot reload occurs, the reassemble method is called.
  @mustCallSuper
  @override
  void reassemble() {
    App.hotLoad = true;
    super.reassemble();
  }

  GlobalKey<NavigatorState> onNavigatorKey() =>
      null; //GlobalKey<NavigatorState>();
  Map<String, WidgetBuilder> onRoutes() => const <String, WidgetBuilder>{};
  String onInitialRoute() => null;
  RouteFactory onOnGenerateRoute() => null;
  RouteFactory onOnUnknownRoute() => null;
  List<NavigatorObserver> onNavigatorObservers() => const <NavigatorObserver>[];
  TransitionBuilder onBuilder() => null;
  String onTitle() => null;
  GenerateAppTitle onOnGenerateTitle() => null;
  Color onColor() => null;
  Locale onLocale() => null;
  // Iterable<LocalizationsDelegate<dynamic>> onLocalizationsDelegates() => null;
  LocaleResolutionCallback onLocaleResolutionCallback() => null;
  // Iterable<Locale> onSupportedLocales() => const <Locale>[Locale('en', 'US')];
  bool onDebugShowMaterialGrid() => false;
  bool onShowPerformanceOverlay() => false;
  bool onCheckerboardRasterCacheImages() => false;
  bool onCheckerboardOffscreenLayers() => false;
  bool onShowSemanticsDebugger() => false;
  bool onDebugShowCheckedModeBanner() => true;

  ThemeData _theme(ThemeData defaultTheme) {
    return ThemeObject.getTheme(defaultTheme);
  }
}

abstract class AppViewState<T extends StatefulWidget> extends StateMVC<T> {
  AppViewState({
    this.con,
    this.user,
    // this.themeObject,
    this.navigatorKey,
    this.routes: const <String, WidgetBuilder>{},
    this.initialRoute,
    this.onGenerateRoute,
    this.onUnknownRoute,
    this.navigatorObservers: const <NavigatorObserver>[],
    this.builder,
    this.title: '',
    this.onGenerateTitle,
    this.color,
    this.theme,
    // this.locale,
    // this.localizationsDelegates,
    this.localeResolutionCallback,
    // this.supportedLocales: const <Locale>[const Locale('en', 'US')],
    this.debugShowMaterialGrid: false,
    this.showPerformanceOverlay: false,
    this.checkerboardRasterCacheImages: false,
    this.checkerboardOffscreenLayers: false,
    this.showSemanticsDebugger: false,
    this.debugShowCheckedModeBanner: true,
    this.debugPaintSizeEnabled = false,
    this.debugPaintBaselinesEnabled = false,
    this.debugPaintPointersEnabled = false,
    this.debugPaintLayerBordersEnabled = false,
    this.debugRepaintRainbowEnabled = false,
  }) : super(con) {
    /// Highlights UI while debugging.
    _debugPaint(
      debugPaintSizeEnabled: debugPaintSizeEnabled,
      debugPaintBaselinesEnabled: debugPaintBaselinesEnabled,
      debugPaintPointersEnabled: debugPaintPointersEnabled,
      debugPaintLayerBordersEnabled: debugPaintLayerBordersEnabled,
      debugRepaintRainbowEnabled: debugRepaintRainbowEnabled,
    );
  }

  final AppController con;
  final UserObject user;

  final GlobalKey<NavigatorState> navigatorKey;
  final Map<String, WidgetBuilder> routes;
  final String initialRoute;
  final RouteFactory onGenerateRoute;
  final RouteFactory onUnknownRoute;
  final List<NavigatorObserver> navigatorObservers;
  final TransitionBuilder builder;
  final String title;
  final GenerateAppTitle onGenerateTitle;
  final ThemeData theme;
  final Color color;
  // final Locale locale;
  //final Iterable<LocalizationsDelegate<dynamic>> localizationsDelegates;
  final LocaleResolutionCallback localeResolutionCallback;
  // final Iterable<Locale> supportedLocales;
  final bool debugShowMaterialGrid;
  final bool showPerformanceOverlay;
  final bool checkerboardRasterCacheImages;
  final bool checkerboardOffscreenLayers;
  final bool showSemanticsDebugger;
  final bool debugShowCheckedModeBanner;

  /// Highlights UI while debugging.
  final bool debugPaintSizeEnabled;
  final bool debugPaintBaselinesEnabled;
  final bool debugPaintPointersEnabled;
  final bool debugPaintLayerBordersEnabled;
  final bool debugRepaintRainbowEnabled;

  /// Provide 'the view'
  Widget build(BuildContext context);

  @mustCallSuper
  Future<bool> init() async {
    bool init = await con.init();
    return init;
  }

  /// Override to dispose anything initialized in your init() function.
  @mustCallSuper
  void dispose() {
    con.dispose();
    super.dispose();
  }

  /// During development, if a hot reload occurs, the reassemble method is called.
  /// This provides an opportunity to reinitialize any data that was prepared
  /// in the initState method.
  @mustCallSuper
  @override
  void reassemble() {
    super.reassemble();
    dispose();
  }
}

class AppController extends ControllerMVC implements AppConMVC {
  AppController([StateMVC state]) : super(state);

  /// Initialize any immediate 'none time-consuming' operations at the very beginning.
  void initApp() {}

  /// Initialize any 'time-consuming' operations at the beginning.
  /// Initialize items essential to the Mobile Applications.
  /// Called by the _App.init() function.
  @mustCallSuper
  Future<bool> init() async {
    await Prefs.init();
    await DeviceInfo.init();

    await WebClient.init(onWebClientBaseUrl(), onWebClientBasicAuth(),
        jwtTokenCallback: () => App.user.token);
    
    String timezone = await FlutterNativeTimezone.getLocalTimezone();
    
    await TimeMachine.initialize(
        {'rootBundle': rootBundle, 'timeZone': timezone});

    await LocaleObject.init();

    await ThemeObject.init();

    await App.user.init();

    return Future.value(true);
  }

  /// Ensure certain objects are 'disposed.'
  /// Callec by the AppState.dispose() function.
  @override
  @mustCallSuper
  void dispose() {
    Prefs.dispose();
    App.user.dispose();

    /// Assets.init(context); called in App.build() -gp
    //Assets.dispose();
    super.dispose();
  }

  WebClientStringCallback onWebClientBaseUrl() => null;
  WebClientStringCallback onWebClientBasicAuth() => null;
  //WebClientStringCallback onWebClientJwtToken() => null;
}

/// High-level function to highlights UI while debugging.
void _debugPaint({
  bool debugPaintSizeEnabled = false,
  bool debugPaintBaselinesEnabled = false,
  bool debugPaintPointersEnabled = false,
  bool debugPaintLayerBordersEnabled = false,
  bool debugRepaintRainbowEnabled = false,
}) {
  /// Highlights UI while debugging.
  debugPaint.debugPaintSizeEnabled = debugPaintSizeEnabled;
  debugPaint.debugPaintBaselinesEnabled = debugPaintBaselinesEnabled;
  debugPaint.debugPaintPointersEnabled = debugPaintPointersEnabled;
  debugPaint.debugPaintLayerBordersEnabled = debugPaintLayerBordersEnabled;
  debugPaint.debugRepaintRainbowEnabled = debugRepaintRainbowEnabled;
}

class _SimpleLoadingScreen extends StatelessWidget {
  const _SimpleLoadingScreen();
  @override
  Widget build(BuildContext context) {
    return Container(
      color: Colors.white,
      child: Center(child: CircularProgressIndicator()),
    );
  }
}
