import 'package:flutter/material.dart';
import 'package:viqcore_community/viqcore_community.dart';

class AnnouncementPage extends StatefulWidget {
  AnnouncementPage({Key key}) : super(key: key);

  static const tag = '/announcement';

  static String title(BuildContext context) =>
      translation(context, LangIds.titleAnnouncement);

  @override
  State createState() => _AnnouncementState();
}

class _AnnouncementState extends StateMVC<AnnouncementPage> {
  _AnnouncementState() : super(AnnouncementController()) {
    con = controller;
  }
  AnnouncementController con;

  @override
  Widget build(BuildContext context) {
    return DataListScaffold(
      dataListController: con,
      listItemBuilder: listItemBuilder,
      listHeaderBuilder : listHeaderBuilder,
      emptyListWidget: Center(
        child: Text("Click the add button to create announcement"),
      ),
      dataFormEditBuilder: dataFormBuilder,
      decoration: Utils.backgroundDecoration("assets/announcement.png"),
    );
  }

Map<String, dynamic> groupby = {};
Widget listHeaderBuilder(BuildContext context, int index) {
  if ( index == 0) groupby.clear();
  DataListController con = DataListController.of(context);
  DataObject dataObject = con.list[index];
  DateTime dateTime = DataUtils.getStartOfDay(dataObject[Announcement.fieldCreatedOn]);
  String group;
  if ( dateTime ==  DataUtils.today ){
    group = 'Today';
  }else if ( dateTime == DataUtils.yesterday){
    group = 'Yesterday';
  } else if (DataUtils.getStartOfWeek(dateTime) == DataUtils.startOfWeek ){
    group = 'This week';
  } else {
    group = 'Last week and before';
  }
  

  if ( !groupby.containsKey(group)){
    groupby[group] = group;
    return Container(child: Text(group),);
  }
  return Container();
  
}

  Widget listItemBuilder(BuildContext context, int index) {
    //DataListController con = ScopedModelObject.of<DataListController>(context);
    DataListController con = DataListController.of(context);
    TextTheme textTheme = Theme.of(context).textTheme;
    DataObject dataObject = con.list[index];

    return Slidable(
      delegate: SlidableScrollDelegate(),
      actionExtentRatio: 0.25,
      secondaryActions: <Widget>[
        IconSlideAction(
          caption: 'Delete',
          color: Colors.red,
          icon: Icons.delete,
          onTap: () => con.delete(index),
        ),
        IconSlideAction(
          caption: 'Edit',
          color: Colors.blue,
          icon: Icons.edit,
          onTap: () => con.openEditForm(context, dataFormBuilder, index: index),
        ),
      ],
      child: Card(
        color: getCardColor(dataObject),
        child: Column(
          children: <Widget>[
            Text(
              dataObject[Announcement.fieldSubject],
              style: textTheme.display1,
            ),
            Text(
              dataObject[Announcement.fieldCreatedOn].toString(),
              style: textTheme.headline,
            ),
            Text(
              dataObject[Announcement.fieldContents],
              style: textTheme.subhead,
            ),
            //DataActionsButton(dataFormBuilder, con, index: index)
          ],
        ),
      ),
    );
  }

  Color getCardColor(DataObject dataObject) {
    return dataObject.get<bool>(Announcement.fieldHasImage, defaultValue: false)
        ? Colors.green
        : Colors.yellow;
  }

  Widget dataFormBuilder(BuildContext context) {
    var con = DataListFormController.of(context);

    List<Widget> fields = [];

    if (con.dataObject != null) {
      fields.add(FormBuilderTextField(
        attribute: "key",
        readonly: true,
        initialValue: con.dataObject['key'],
        decoration: InputDecoration(
          labelText: "Key",
          icon: Icon(Icons.vpn_key),
        ),
      ));
    }

    fields.add(FormBuilderTextField(
      attribute: "subject",
      readonly: con.inAsyncCall,
      initialValue: con.dataObject?.get<String>('subject', defaultValue: ''),
      decoration: InputDecoration(
        labelText: "Subject",
        icon: Icon(Icons.title),
      ),
      validators: [
        FormBuilderValidators.required(),
      ],
    ));

    fields.add(FormBuilderTextField(
      attribute: "contents",
      readonly: con.inAsyncCall,
      initialValue: con.dataObject?.get<String>('contents', defaultValue: ''),
      maxLines: 5,
      decoration: InputDecoration(
        labelText: "Contents",
        icon: Icon(Icons.text_fields),
      ),
      validators: [
        FormBuilderValidators.required(),
      ],
    ));

    return Padding(
      padding: EdgeInsets.all(10),
      child: SingleChildScrollView(
        child: FormBuilder(
          context,
          key: con.formKey,
          autovalidate: false,
          // readonly: true,

          child: Column(children: fields),
        ),
      ),
    );
  }
}
