import 'package:flutter/widgets.dart';
import 'package:viqcore_community/viqcore_community.dart';

class User extends UserObject {
  static const String fieldUserId = 'userId';
  static const String fieldName = 'name';
  static const String fieldPhoneNumber = 'phoneNumber';
  static const String fieldIsOneTimePassword = 'isOneTimePassword';
  static const String fieldcommunityKey = 'communityKey';
  static const String fieldAdminLevel = 'adminLevel';
  static const String fieldIsActivated = 'isActivated';
  static const String fieldHasImage = 'hasImage';
  static const String fieldHouseNr = 'houseNr';
  static const String fieldStreet = 'street';
  static const String fieldToken = 'token';

  static const String fieldImageDataBase64 = 'imageDataBase64';

  String get name => this[fieldName];
  String get userId => this[fieldUserId];
  bool get hasImage => this.get<bool>(fieldHasImage, defaultValue: false);
  String get imageUrl => hasImage ? Utils.imageUrl(userId) : null;

  User() {
    this.valueChangedCallback = {
      fieldToken: (val) => token = val,
    };
  }

  static User of(BuildContext context){
    return Provider.of<UserObject>(context) as User;
  }
 
  @override
  Future<WebClientResult> onLoginToken() async {
    final String api = '/api/Login/AutoLogin';
    var data = {};
    return await WebClient.current.post(api, data);
  }

  @override
  Future<WebClientResult> onLoginUserPassword(
      String userId, String password) async {
    final String api = '/api/Login';
    var data = {};
    data[fieldUserId] = userId;
    data['password'] = password;
    return await WebClient.current.post(api, data);
  }

  @override
  void onLoginSuccess(WebClientResult result) {
    this.fill(result.data);
    super.onLoginSuccess(result);
  }


  Future<DataResponse> updateImage(String imageDataBase64) async {
    final String api = '/api/User/UpdateImage';
    var data = {};
    data[fieldUserId] = this.userId;
    data[fieldImageDataBase64] = imageDataBase64;
    WebClientResult result = await WebClient.current.post(api, data);
    if ( result.ok){
      fill(result.data);
    }
    return DataResponse.fromWebClientResult(result);
  }

  // @override
  // void onFieldValue(String fieldName, value) {
  //   super.onFieldValue(fieldName, value);
  //   if (fieldName == fieldToken) {
  //     token = value;
  //   }
  // }
}
